@@ -462,25 +462,59 @@ void Document_Glom::set_tables(const type_listTableInfo& tables)
       info.m_hidden = iterfind->m_hidden;
       info.m_title = iterfind->m_title;
       info.m_default = iterfind->m_default;
-      
+
       something_changed = true;
     }
   }
 
   if(something_changed)
     set_modified();
-  
+
 }
 
 Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_relationship_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Relationship& relationship) const
 {
   //TODO: Use an actual relationship_name instead of concatenating:
-  return get_data_layout_groups_plus_new_fields(layout_name, relationship.get_from_table() + "_related_" + relationship.get_name()); 
+  return get_data_layout_groups_plus_new_fields(layout_name + "_related_" + relationship.get_name(), relationship.get_from_table(), relationship.get_to_table());
 }
 
-Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& table_name) const
+void Document_Glom::fill_layout_field_details(const Glib::ustring& parent_table_name, LayoutGroup& layout_group) const
 {
-  type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, table_name);
+  //Get the full field information for the LayoutItem_Fields in this group:
+
+  for(LayoutGroup::type_map_items::iterator iter = layout_group.m_map_items.begin(); iter != layout_group.m_map_items.end(); ++iter)
+  {
+    LayoutItem* layout_item = iter->second;
+
+    LayoutItem_Field* layout_field = dynamic_cast<LayoutItem_Field*>(layout_item);
+    if(layout_field)
+    {
+      get_field(parent_table_name, layout_field->get_name(), layout_field->m_field);
+    }
+    else
+    {
+      LayoutGroup* layout_group_child = dynamic_cast<LayoutGroup*>(layout_item);
+      if(layout_group_child)
+        fill_layout_field_details(parent_table_name, *layout_group_child); //recurse
+    }
+  }
+}
+
+void Document_Glom::fill_layout_field_details(const Glib::ustring& parent_table_name, type_mapLayoutGroupSequence& sequence) const
+{
+  for(type_mapLayoutGroupSequence::iterator iterGroups = sequence.begin(); iterGroups != sequence.end(); ++iterGroups)
+  {
+    fill_layout_field_details(parent_table_name, iterGroups->second);
+  }
+}
+
+Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const Glib::ustring& table_name) const
+{
+  Glib::ustring child_table_name = table_name;
+  if(child_table_name.empty())
+    child_table_name = parent_table_name;
+
+  type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, parent_table_name);
 
 
   //If there are no fields in the layout, then add a default:
@@ -491,6 +525,8 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
 
   if(create_default)
   {
+    g_warning("Document_Glom::get_data_layout_groups_plus_new_field(): Creating default layout for table %s, for layout %s", parent_table_name.c_str(), layout_name.c_str());
+
     //Get the last top-level group. We will add new fields into this one:
     //TODO_Performance: There must be a better way to do this:
     LayoutGroup* pTopLevel = 0;
@@ -511,7 +547,7 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
     }
 
     //Discover new fields, and add them:
-    type_vecFields all_fields = get_table_fields(table_name);
+    type_vecFields all_fields = get_table_fields(child_table_name);
     for(type_vecFields::const_iterator iter = all_fields.begin(); iter != all_fields.end(); ++iter)
     {
       const Glib::ustring field_name = iter->get_name();
@@ -532,8 +568,8 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
         if(!found)
         {
           LayoutItem_Field layout_item;
-          layout_item.set_name(field_name);
-          //layout_item.set_table_name(table_name); //TODO: Allow viewing of fields through relationships.
+          layout_item.m_field = *iter;
+          //layout_item.set_table_name(child_table_name); //TODO: Allow viewing of fields through relationships.
           //layout_item.m_sequence = sequence;  add_item() will fill this.
 
 
@@ -567,7 +603,7 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
 void Document_Glom::set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups)
 {
   //TODO: Use an actual relationship_name instead of concatenating:
-  set_data_layout_groups(layout_name, relationship.get_from_table() + "_related_" + relationship.get_name(), groups);
+  set_data_layout_groups(layout_name + "_related_" + relationship.get_name(), relationship.get_from_table(), groups);
 }
 
 void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name, const type_mapLayoutGroupSequence& groups)
@@ -752,9 +788,22 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
         LayoutItem_Field item;
 
         item.set_name( get_node_attribute_value(element, "name") );
-        item.set_relationship_name( get_node_attribute_value(element, "relationship") );
+
+        const Glib::ustring relationship_name = get_node_attribute_value(element, "relationship"); 
+        //item.set_relationship_name(relationship_name);
+        if(!relationship_name.empty())
+          get_relationship(table_name, relationship_name, item.m_relationship);
+
         item.set_editable( get_node_attribute_value_as_bool(element, "editable") );
-        //item.set_table_name(table_name);
+
+        //Numeric formatting:
+        NumericFormat numeric_format;
+        numeric_format.m_use_thousands_separator = get_node_attribute_value_as_bool(element, "format_thousands_separator");
+        numeric_format.m_decimal_places_restricted = get_node_attribute_value_as_bool(element, "format_decimal_places_restricted");
+        numeric_format.m_decimal_places = get_node_attribute_value_as_decimal(element, "format_decimal_places");
+        numeric_format.m_currency_symbol = get_node_attribute_value(element, "format_currency_symbol");
+
+        item.m_numeric_format = numeric_format;
 
         item.m_sequence = sequence;
         group.add_item(item, sequence);
@@ -1003,7 +1052,7 @@ bool Document_Glom::load_after()
       }
     }
   }
-  
+
   return result;
 }
 
@@ -1038,6 +1087,12 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
         nodeItem->set_attribute("relationship", field->get_relationship_name());
         set_node_attribute_value_as_bool(nodeItem, "editable", item->get_editable());
 
+        //Numeric format:
+        set_node_attribute_value_as_bool(nodeItem, "format_thousands_separator",  field->m_numeric_format.m_use_thousands_separator);
+        set_node_attribute_value_as_bool(nodeItem, "format_decimal_places_restricted", field->m_numeric_format.m_decimal_places_restricted);
+        set_node_attribute_value_as_decimal(nodeItem, "format_decimal_places", field->m_numeric_format.m_decimal_places);
+        set_node_attribute_value(nodeItem, "format_currency_symbol", field->m_numeric_format.m_currency_symbol);
+
         set_node_attribute_value_as_decimal(nodeItem, "sequence", item->m_sequence);
       }
       else
@@ -1106,7 +1161,11 @@ bool Document_Glom::save_before()
           set_node_attribute_value_as_bool(elemField, "primary_key", field_info.get_primary_key());
           set_node_attribute_value_as_bool(elemField, "unique", field_info.get_unique_key());
           set_node_attribute_value_as_bool(elemField, "auto_increment", field_info.get_auto_increment());
-          set_node_attribute_value(elemField, "default_value", field_info.get_default_value().to_string());
+
+          NumericFormat format_ignored; //Because we use ISO format.
+          const Glib::ustring default_value_as_text = GlomConversions::get_text_for_gda_value(field.get_glom_type(), field_info.get_default_value(), std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
+          set_node_attribute_value(elemField, "default_value", default_value_as_text);
+
 
           set_node_attribute_value(elemField, "calculation", field.get_calculation());
 
@@ -1149,6 +1208,7 @@ bool Document_Glom::save_before()
         xmlpp::Element* nodeDataLayouts = nodeTable->add_child("data_layouts");
 
         //Add the groups:
+        //Make sure that we always get these _after_ the relationships.
         for(DocumentTableInfo::type_layouts::const_iterator iter = doctableinfo.m_layouts.begin(); iter != doctableinfo.m_layouts.end(); ++iter)
         {
           xmlpp::Element* nodeLayout = nodeDataLayouts->add_child("data_layout");
