@@ -23,8 +23,10 @@
 #include "../data_structure/layout/layoutitem_field.h"
 #include "../python_embed/glom_python.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <algorithm> //For std::find()
+#include <libxslt/transform.h>
 #include "config.h"
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
 Box_Data::Box_Data()
 : m_Button_Find(Gtk::Stock::FIND),
@@ -100,6 +102,50 @@ void Box_Data::on_Button_Find()
   signal_find.emit(get_WhereClause());
 }
 
+Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Value& primary_key_value)
+{
+  type_map_fields field_values;
+
+  Field fieldPrimaryKey;
+  get_field_primary_key(fieldPrimaryKey);
+
+  Document_Glom* document = get_document();
+  if(document)
+  {
+    //TODO: Cache the list of all fields, as well as caching (m_Fields) the list of all visible fields:
+    const Document_Glom::type_vecFields fields = document->get_table_fields(m_strTableName);
+
+    //TODO: This seems silly. We should just have a build_sql_select() that can take this container:
+    type_vecLayoutFields fieldsToGet;
+    for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+    {
+      LayoutItem_Field layout_item;
+      layout_item.m_field = *iter;
+      fieldsToGet.push_back(layout_item);
+    }
+
+    const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, fieldPrimaryKey, primary_key_value);
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(query);
+
+    if(data_model && data_model->get_n_rows())
+    {
+      int col_index = 0;
+      for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+      {
+        //There should be only 1 row. Well, there could be more but we will ignore them.
+        field_values[iter->get_name()] = data_model->get_value_at(col_index, 0);
+        ++col_index;
+      }
+    }
+    else
+    {
+      handle_error();
+    }
+  }
+
+  return field_values;
+}
+
 Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value)
 {
   Field fieldPrimaryKey;
@@ -114,19 +160,19 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
   if(iterFindPrimary == fieldsToAdd.end())
   {
     LayoutItem_Field layout_item;
-    layout_item.set_name(primary_key_name);
     layout_item.m_field = fieldPrimaryKey;
 
     fieldsToAdd.push_back(layout_item);
   }
 
   //Calculate any necessary field values and enter them:
-  for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.begin(); ++iter)
+  for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
   {
     const LayoutItem_Field& layout_item = *iter;
 
     //If the user did not enter something in this field:
     Gnome::Gda::Value value = get_entered_field_data(layout_item);
+
     if(GlomConversions::value_is_empty(value)) //This deals with empty strings too.
     {
       const Field& field = layout_item.m_field;
@@ -135,9 +181,24 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
       const Glib::ustring calculation = field.get_calculation(); //TODO_Performance: Use a get_has_calculation() method.
       if(!calculation.empty())
       {
-        Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), calculation);
+        const type_map_fields field_values = get_record_field_values(primary_key_value);
+
+        const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), calculation, field_values,
+          get_document(), m_strTableName);
         set_entered_field_data(layout_item, value);
       }
+
+      //Use default values (These are also specified in postgres as part of the field definition,
+      //so we could theoretically just not specify it here.
+      //TODO_Performance: Add has_default_value()?
+      if(GlomConversions::value_is_empty(value))
+      {
+        const Gnome::Gda::Value default_value = field.get_field_info().get_default_value();
+        if(!GlomConversions::value_is_empty(default_value))
+        {
+          set_entered_field_data(layout_item, default_value);
+        }
+      }
     }
   }
 
@@ -224,8 +285,8 @@ bool Box_Data::confirm_discard_unstored_data() const
   if(get_unstored_data())
   {
     //Ask user to confirm loss of data:
-    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(gettext("No primary key value")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL );
-    dialog.set_secondary_text(gettext("This data can not be stored in the database because you have not provided a primary key.\nDo you really want to discard this data?"));
+    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL );
+    dialog.set_secondary_text(_("This data can not be stored in the database because you have not provided a primary key.\nDo you really want to discard this data?"));
     //TODO: dialog.set_transient_for(*this);
     int iButton = dialog.run();
 
@@ -374,7 +435,6 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
       if(bPrimaryKeyFound)
       {
         LayoutItem_Field layout_item;
-        layout_item.set_name(primary_key_field_name);
         layout_item.m_field = all_fields[iPrimaryKey];
 
         //Don't use thousands separators with ID numbers:
@@ -397,7 +457,6 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
         if(iter->get_name() != primary_key_field_name) //We already added the primary key.
         {
           LayoutItem_Field layout_item;
-          layout_item.set_name(iter->get_name());
           layout_item.m_field = field_info;
 
           layout_item.set_editable(true); //A sensible default.
@@ -497,6 +556,29 @@ Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring&
   return result;
 }
 
+Box_Data::type_vecLayoutFields Box_Data::get_calculated_fields(const Glib::ustring& field_name) const
+{
+  type_vecLayoutFields result;
+
+  const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
+  if(document)
+  {
+    //TODO: examine all fields, not just the the shown fields (m_Fields):
+    for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end();  ++iter)
+    {
+      const Field& field = iter->m_field;
+
+      //Does this field's calculation use the field?
+      const Field::type_list_strings fields_triggered = field.get_calculation_fields();
+      Field::type_list_strings::const_iterator iterFind = std::find(fields_triggered.begin(), fields_triggered.end(), field_name);
+      if(iterFind != fields_triggered.end())
+        result.push_back(*iter);
+    }
+  }
+
+  return result;
+}
+
 /** Get the fields whose values should be looked up when @a field_name changes, with
  * the relationship used to lookup the value.
  */
@@ -507,6 +589,7 @@ Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& fie
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
   if(document)
   {
+    //TODO: examine all fields, not just the the shown fields (m_Fields):
     for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end();  ++iter)
     {
       const Field& field = iter->m_field;
@@ -560,7 +643,94 @@ Gnome::Gda::Value Box_Data::get_lookup_value(const Relationship& relationship, c
 
   return result;
 }
-  
+
+void Box_Data::do_calculations(const LayoutItem_Field& field_changed, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+{
+  //TODO: Get (recursively) the list of fields to recalcualte, sort them in order of dependency, then calculate them all.
+
+  //g_warning("Box_Data::do_calculations(): field_changed=%s", field_changed.get_name().c_str());
+
+  //TODO_Performance: This is profoundly inefficient:
+
+  //Recalculate fields that are triggered by a change of this field's value:
+  const type_vecLayoutFields calculated_fields = get_calculated_fields(field_changed.get_name());
+  for(type_vecLayoutFields::const_iterator iter = calculated_fields.begin(); iter != calculated_fields.end(); ++iter)
+  {
+    const LayoutItem_Field& field = *iter;
+
+    type_vecLayoutFields::const_iterator iterFind = std::find(m_FieldsCalculationInProgress.begin(), m_FieldsCalculationInProgress.end(), field);
+    if(iterFind != m_FieldsCalculationInProgress.end())
+    {
+      g_warning("Box_Data::do_calculations(): Circular calculation detected. The Infinite loop was prevented before re-calculating field %s", field.get_name().c_str());
+    }
+    else
+    {
+      //g_warning("CircularPrevention: recalculating field=%s", field.get_name().c_str());
+
+      //recalculate:
+      const type_map_fields field_values = get_record_field_values(primary_key_value);
+      Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.m_field.get_glom_type(), iter->m_field.get_calculation(), field_values,
+            get_document(), m_strTableName);
+
+      //show it:
+      set_entered_field_data(*iter, value);
+
+      //Add it to the database (even if it is not shown in the view)
+      set_field_value_in_database(*iter, value, primary_key, primary_key_value); //This triggers other recalculations/lookups.
+    }
+  }
+}
+
+bool Box_Data::set_field_value_in_database(const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value)
+{
+  return set_field_value_in_database(Gtk::TreeModel::iterator(), field_layout, field_value, primary_key_field, primary_key_value);
+}
+
+bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value)
+{
+  //row is invalid, and ignored, for Box_Data_Details.
+
+  const Field& field = field_layout.m_field;
+
+  const Glib::ustring field_name = field_layout.get_name();
+  if(!field_name.empty()) //This should not happen.
+  {
+    Glib::ustring strQuery = "UPDATE " + m_strTableName;
+    strQuery += " SET " + field_name + " = " + field.sql(field_value);
+    strQuery += " WHERE " + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(strQuery);  //TODO: Handle errors
+    if(!datamodel)
+    {
+      g_warning("Box_Data::set_field_value_in_database(): UPDATE failed.");
+      return false; //failed.
+    }
+
+    //Get-and-set values for lookup fields, if this field triggers those relationships:
+    do_lookups(row, field_layout, field_value, primary_key_field, primary_key_value);
+
+    //Update related fields, if this field is used in the relationship:
+    refresh_related_fields(row, field_layout, field_value, primary_key_field, primary_key_value);
+
+    //Calculate any dependent fields.
+    //TODO: Make lookups part of this?
+    //Prevent circular calculations during the recursive do_calculations:
+    {
+      m_FieldsCalculationInProgress.push_back(field_layout);
+
+      //Recalculate any calculated fields that depend on this calculated field.
+      do_calculations(field_layout, primary_key_field, primary_key_value);
+
+      type_vecLayoutFields::iterator iterFind = std::find(m_FieldsCalculationInProgress.begin(), m_FieldsCalculationInProgress.end(), field_layout);
+      if(iterFind != m_FieldsCalculationInProgress.end())
+      {
+        m_FieldsCalculationInProgress.erase(iterFind);
+      }
+    }
+  }
+
+  return true;
+}
+
 Document_Glom::type_mapLayoutGroupSequence Box_Data::get_data_layout_groups(const Glib::ustring& layout)
 {
   Document_Glom::type_mapLayoutGroupSequence layout_groups;
@@ -729,12 +899,13 @@ Glib::ustring Box_Data::build_sql_select(const Glib::ustring& table_name, const
   if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a record to show:
   {
     const Glib::ustring where_clause = m_strTableName + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
-    return build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
+    return util_build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
   }
 
   return Glib::ustring();
 }
 
+/*
 Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause)
 {
   Glib::ustring result;
@@ -797,6 +968,7 @@ Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring&
 
   return result;
 }
+*/
 
 Glib::ustring Box_Data::get_layout_item_table_name(const LayoutItem_Field& layout_item, const Glib::ustring table_name)
 {
@@ -854,8 +1026,8 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
     {
       //Warn the user:
       //TODO: Make the field insensitive until it can receive data, so people never see this dialog.
-      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(gettext("Related Record Does Not Exist")), true);
-      dialog.set_secondary_text(gettext("Data may not be entered into this related field, because the related record does not yet exist, and the relationship does not allow automatic creation of new related records."));
+      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Related Record Does Not Exist")), true);
+      dialog.set_secondary_text(_("Data may not be entered into this related field, because the related record does not yet exist, and the relationship does not allow automatic creation of new related records."));
       dialog.set_transient_for(*get_app_window());
       dialog.run();
 
@@ -873,9 +1045,9 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
       {
         //Warn the user:
         //TODO: Make the field insensitive until it can receive data, so people never see this dialog.
-        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(gettext("Related Record Can Not Be Created")), true);
+        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Related Record Can Not Be Created")), true);
         //TODO: This is a very complex error message:
-        dialog.set_secondary_text(gettext("Data may not be entered into this related field, because the related record does not yet exist, and the key in the related record is auto-generated and therefore can not be created with the key value in this record."));
+        dialog.set_secondary_text(_("Data may not be entered into this related field, because the related record does not yet exist, and the key in the related record is auto-generated and therefore can not be created with the key value in this record."));
         dialog.set_transient_for(*get_app_window());
         dialog.run();
 
@@ -937,3 +1109,53 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
 
   return true;
 }
+
+void Box_Data::print_layout()
+{
+  Gtk::MessageDialog dialog("<b>Not implemented</b>", true);
+  dialog.set_secondary_text("Sorry, this feature has not been implemented yet.");
+  dialog.set_transient_for(*get_app_window());
+  dialog.run();
+}
+
+Glib::ustring Box_Data::xslt_process(const xmlpp::Document& xml_document, const std::string& filepath_xslt)
+{
+  Glib::ustring  result;
+
+  //Use libxslt to transform the XML:
+  xmlDocPtr style = xmlReadFile(filepath_xslt.c_str(), 0, 0);
+  if(style)
+  {
+    //Parse the stylesheet:
+    xsltStylesheetPtr cur = xsltParseStylesheetDoc(style);
+    if(cur)
+    {
+      //Use the parsed stylesheet on the XML:
+      xmlDocPtr pDocOutput = xsltApplyStylesheet(cur, const_cast<xmlDoc*>(xml_document.cobj()), 0);
+      xsltFreeStylesheet(cur);
+
+      //Get the output text:
+      xmlChar* buffer = 0;
+      int length = 0;
+      xmlDocDumpFormatMemoryEnc(pDocOutput, &buffer, &length, 0, 0);
+
+      if(buffer)
+      {
+        // Create a Glib::ustring copy of the buffer
+
+        // Here we force the use of Glib::ustring::ustring( InputIterator begin, InputIterator end )
+        // instead of Glib::ustring::ustring( const char*, size_type ) because it
+        // expects the length of the string in characters, not in bytes.
+        result = Glib::ustring( reinterpret_cast<const char *>(buffer), reinterpret_cast<const char *>(buffer + length) );
+
+        // Deletes the original buffer
+        xmlFree(buffer);
+      }
+
+      xmlFreeDoc(pDocOutput);
+    }
+  }
+
+  return result;
+}
+
