@@ -33,6 +33,8 @@ public:
   virtual void init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause = Glib::ustring());
   virtual void refresh_db_details(const Glib::ustring& strWhereClause = Glib::ustring());
 
+  virtual void print_layout(); //A test, for now.
+
   typedef std::vector<LayoutItem_Field> type_vecLayoutFields;
 
   virtual Glib::ustring get_WhereClause() const;
@@ -57,20 +59,23 @@ public:
 protected:
   virtual void fill_from_database(); //override.
 
+  virtual void do_lookups(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value) = 0;
+  virtual void refresh_related_fields(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value) = 0;
+
   virtual type_vecLayoutFields get_fields_to_show() const;
   virtual Glib::ustring build_sql_select(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value);
-  virtual Glib::ustring build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause);
+  //virtual Glib::ustring build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause);
   virtual bool get_related_record_exists(const Relationship& relationship, const Field& key_field, const Gnome::Gda::Value& key_value);
   virtual bool add_related_record_for_field(const LayoutItem_Field& layout_item_parent, const Relationship& relationship, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value_provided);
 
   type_vecLayoutFields get_table_fields_to_show(const Glib::ustring& table_name) const;
   type_vecLayoutFields get_table_fields_to_show(const Glib::ustring& table_name, const Document_Glom::type_mapLayoutGroupSequence& mapGroupSequence) const;
-  void get_table_fields_to_show_add_group(const Glib::ustring& table_name, const type_vecFields& all_db_fields, const LayoutGroup& group, Box_Data::type_vecLayoutFields& vecFields) const;
+  void get_table_fields_to_show_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const LayoutGroup& group, Box_Data::type_vecLayoutFields& vecFields) const;
 
   /** Get the layout groups, with the Field information filled in.
    */
   Document_Glom::type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout);
-  void fill_layout_group_field_info(LayoutGroup& group);
+  void fill_layout_group_field_info(LayoutGroup& group, const Privileges& table_privs);
 
   typedef std::pair<LayoutItem_Field, Relationship> type_pairFieldTrigger;
   typedef std::list<type_pairFieldTrigger> type_list_lookups;
@@ -80,6 +85,10 @@ protected:
    */
   type_list_lookups get_lookup_fields(const Glib::ustring& field_name) const;
 
+  /** Get the fields whose values should be recalculated when @a field_name changes.
+   */
+  type_vecLayoutFields get_calculated_fields(const Glib::ustring& field_name) const;
+
   /** Get the fields that are in related tables, via a relationship using @a field_name changes.
   */
   type_vecLayoutFields get_related_fields(const Glib::ustring& field_name) const;
@@ -88,6 +97,10 @@ protected:
    */
   Gnome::Gda::Value get_lookup_value(const Relationship& relationship, const Field& source_field, const Gnome::Gda::Value & key_value);
 
+  virtual void do_calculations(const LayoutItem_Field& field_changed, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
+  bool set_field_value_in_database(const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
+  bool set_field_value_in_database(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
+
   virtual bool record_delete(const Gnome::Gda::Value& primary_key_value);
   virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); //New record with all entered field values.
   Gnome::Gda::Value generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name);
@@ -105,6 +118,13 @@ protected:
   static bool get_field_primary_key_index(const type_vecFields& fields, guint& field_column);
   static bool get_field_primary_key_index(const type_vecLayoutFields& fields, guint& field_column);
 
+  typedef std::map<Glib::ustring, Gnome::Gda::Value> type_map_fields;
+  //TODO: Performance: This is massively inefficient:
+  type_map_fields get_record_field_values(const Gnome::Gda::Value& primary_key_value);
+
+
+  static Glib::ustring xslt_process(const xmlpp::Document& xml_document, const std::string& filepath_xslt);
+
   Gtk::Button m_Button_Find; //only used by _Find sub-classes. Should be MI.
   Gtk::Label m_Label_FindStatus;
 
@@ -116,6 +136,8 @@ protected:
   Glib::ustring m_strWhereClause;
 
   type_vecLayoutFields m_Fields;
+
+  type_vecLayoutFields m_FieldsCalculationInProgress; //Prevent circular calculations.
 };
 
 #endif
