@@ -32,40 +32,43 @@ public:
   Box_Data_Details(bool bWithNavButtons = true);
   virtual ~Box_Data_Details();
 
-  
   virtual void init_db_details(const Glib::ustring& strTableName, const Gnome::Gda::Value& primary_key_value);
   virtual void refresh_db_details(const Gnome::Gda::Value& primary_key_value);
   virtual void refresh_db_details_blank();
-  
+
+  virtual void print_layout();
+
   virtual Gnome::Gda::Value get_primary_key_value() const; //Actual primary key value of this record.
   virtual Gnome::Gda::Value get_primary_key_value_selected(); //Value in the primary key's cell.
-    
-  virtual Gnome::Gda::Value get_entered_field_data(const Field& field) const;
-  virtual void set_entered_field_data(const Field& field, const Gnome::Gda::Value& value);
-  
+
+  virtual Gnome::Gda::Value get_entered_field_data(const LayoutItem_Field& field) const;
+  virtual void set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
+
   //Signals:
-  
   typedef sigc::signal<void> type_signal_void;
   type_signal_void signal_nav_first();
   type_signal_void signal_nav_prev();
   type_signal_void signal_nav_next();
   type_signal_void signal_nav_last();
-  
+
   typedef sigc::signal<void, Gnome::Gda::Value> type_signal_record_deleted; //arg is PrimaryKey.   //TODO: pass by const ref?
   type_signal_record_deleted signal_record_deleted();
-  
+
   //typedef sigc::signal<void, Glib::ustring, Gnome::Gda::Value> type_signal_user_requested_related_details; //args are TableName, PrimaryKey.
   //type_signal_user_requested_related_details signal_user_requested_related_details();
-   
+
 protected:
   virtual void fill_from_database(); //override.
   virtual void fill_from_database_layout();
   //virtual void fill_related();
 
-  void do_lookups(const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
+  virtual void do_lookups(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
+  virtual void refresh_related_fields(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
 
   virtual bool get_field_primary_key(Field& field) const;
 
+  void print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group);
+
   //Signal handlers:
   virtual void on_button_new();
   virtual void on_button_del();
@@ -77,14 +80,17 @@ protected:
 
   virtual void on_userlevel_changed(AppState::userlevels user_level); //override
   virtual void on_flowtable_layout_changed();
-  
+
   //Signal handler: The last 2 args are bind-ed.
   virtual void on_related_record_added(Gnome::Gda::Value key_value, Glib::ustring strFromKeyName);
 
   //Signal handler: The last arg is bind-ed.
   //virtual void on_related_user_requested_details(Gnome::Gda::Value key_value, Glib::ustring strTableName);
 
-  virtual void on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& value);
+  virtual void on_flowtable_field_edited(const LayoutItem_Field& id, const Gnome::Gda::Value& value);
+  virtual void on_flowtable_related_record_changed(const Glib::ustring& relationship_name);
+
+  virtual void recalculate_fields_for_related_records(const Glib::ustring& relationship_name);
 
   Field m_field_primary_key;
   Gnome::Gda::Value m_primary_key_value;
@@ -101,7 +107,7 @@ protected:
   Gtk::Label m_Label_Related;
   Gtk::Notebook m_Notebook_Related;
   */
-  
+
   Gtk::Button m_Button_Nav_First;
   Gtk::Button m_Button_Nav_Prev;
   Gtk::Button m_Button_Nav_Next;
